# Copyright (c) 2016 - for information on the respective copyright owner
# see the NOTICE file and/or the repository https://github.com/boschresearch/assets2036py.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
from enum import Enum
from os import path
import json
import ssl
import traceback
from json import JSONDecodeError
from urllib.request import urlopen
from abc import ABC, abstractmethod
from numbers import Number
import jsonschema
from assets2036py.exceptions import InvalidParameterException, NotWritableError
from .utilities import sanitize

# pylint: disable=protected-access, line-too-long


logger = logging.getLogger(__name__)
schema_file = path.abspath(path.dirname(__file__)) + \
    "/resources/submodel_schema.json"

with open(schema_file) as f:
    submodel_schema = json.load(f)

# pylint: disable=no-member


class PropertyType(Enum):
    BOOLEAN = 0
    INTEGER = 1
    NUMBER = 2
    STRING = 3
    OBJECT = 5
    NULL = 6
    ARRAY = 7

    def native_type(self):
        type_map = {
            self.BOOLEAN: bool,
            self.INTEGER: int,
            self.NUMBER: Number,
            self.OBJECT: dict,
            self.STRING: str,
            self.ARRAY: list
        }
        if self in type_map:
            return type_map[self]
        else:
            raise Exception(f"No native Type for {self.name}")


class Mode(Enum):
    OWNER = 0
    CONSUMER = 1


class Property(ABC):
    def __init__(self, name, parent, property_definition):
        self._val = None
        self.schema = property_definition
        self.property_type = PropertyType[property_definition["type"].upper()]
        self.parent = parent
        self.communication_client = parent.communication_client
        self.name = name
        self.callbacks = []

    # noinspection PyProtectedMember

    def _get_topic(self):
        return self.parent._get_topic() + f"/{self.name}"

    def assert_valid_type(self, value):
        jsonschema.validate(value, self.schema)

    @property
    @abstractmethod
    def value(self):
        return self._val

    @value.setter
    @abstractmethod
    def value(self, value):

        pass

    def on_change(self, callback):
        self.callbacks.append(callback)


class WritableProperty(Property):

    @property
    def value(self):
        return self._val

    @value.setter
    def value(self, value):
        self.assert_valid_type(value)
        self._val = value
        self.communication_client.publish(
            self._get_topic(), json.dumps(value))

    def delete(self):
        logger.debug("deleting %s", self.name)
        self.communication_client.publish(self._get_topic(), "")


class ReadOnlyProperty(Property):

    def __init__(self, name, parent, property_definition):
        super().__init__(name, parent, property_definition)
        self.communication_client.subscribe(self._get_topic(), self._update)

    @property
    def value(self):
        return self._val

    @value.setter
    def value(self, value):
        raise NotWritableError("Writing to a ReadonlyProperty not allowed.")

    def _update(self, payload):
        logger.debug("Updating %s to %s", self.name, payload)
        try:
            updated_value = json.loads(payload)
            self.assert_valid_type(updated_value)
            self._val = updated_value
            for cb in self.callbacks:
                cb(self._val)
        except JSONDecodeError as jsonexc:
            logger.error("Payload '%s' was invalid JSON: %s", payload, jsonexc)
        except AssertionError as assertexc:
            logger.error("Parsed payload '%s' has wrong type: %s",
                         payload, assertexc)


class Event(ABC):
    def __init__(self, name, parent, event_definition):
        self.name = name
        self.parent = parent
        self.communication_client = parent.communication_client
        if "parameters" in event_definition:
            self._parameters = {name: PropertyType[schema["type"].upper()] for name, schema in
                                event_definition["parameters"].items()}

    def _validate_parameters(self, params):
        if params.keys() == self._parameters.keys():
            return all(isinstance(val, self._parameters[name].native_type()) for name, val in params.items())
        return False

    def _get_topic(self):
        return self.parent._get_topic() + f"/{self.name}"


class SubscribableEvent(Event):

    def on_event(self, callback):
        def callback_func(parameters, timestamp):
            if not self._validate_parameters(parameters):
                raise InvalidParameterException(
                    f" expected {','.join(self._parameters.keys())} but received {parameters}")
            callback(timestamp, **parameters)

        self.communication_client.subscribe_event(
            self._get_topic(), callback_func)


class TriggerableEvent(Event):

    def trigger(self, **params):
        if not self._validate_parameters(params):
            raise InvalidParameterException(
                f" expected {self._parameters.keys()} but got {params}")
        self.communication_client.trigger_event(self._get_topic(), params)


class Operation(ABC):
    def __init__(self, name, parent, operation_definition):
        self.name = name
        self.parent = parent
        self.communication_client = parent.communication_client
        self._response_schema = None
        self._parameters = {}
        if "parameters" in operation_definition:
            self._parameters = {name: PropertyType[schema["type"].upper()] for name, schema in
                                operation_definition["parameters"].items()}
        if "response" in operation_definition:
            self._response_type = PropertyType[operation_definition["response"]["type"].upper(
            )]
            self._response_schema = operation_definition["response"]

    @abstractmethod
    def invoke(self, timeout=None, **params):
        raise NotImplementedError("You can only call remote operations..")

    def _validate_parameters(self, params):
        if params.keys() == self._parameters.keys():
            return all(isinstance(val, self._parameters[name].native_type()) for name, val in params.items())
        return False

    # noinspection PyProtectedMember
    def _get_topic(self):
        return self.parent._get_topic() + f"/{self.name}"


class CallableOperation(Operation):

    def invoke(self, timeout=None, **params):
        if not self._validate_parameters(params):
            raise InvalidParameterException(
                f" expected {self._parameters} but got {params}")
        logger.debug("Calling %s", self.name)
        res = self.communication_client.invoke_operation(
            self._get_topic(), params, timeout)
        logger.debug("%s got %s", self.name, res)
        if hasattr(self, "_response_type"):
            assert isinstance(
                res, self._response_type.native_type())
            return res


class BindableOperation(Operation):
    def invoke(self, timeout=None, **params):
        super().invoke(timeout=timeout, **params)

    def bind(self, callback):
        def callback_func(parameters):
            if not self._validate_parameters(parameters):
                raise InvalidParameterException(
                    f" expected list with {self._parameters.keys()} but received {parameters}")
            try:
                res = callback(**parameters)
                logger.debug("callback executed, response is %s", res)
            # pylint: disable=broad-except
            except Exception:
                logger.error(
                    "operation '%s' caused exception:\n%s", callback, traceback.format_exc())
            if hasattr(self, "_response_type"):
                jsonschema.validate(res, self._response_schema)
                return res

        self.communication_client.bind_operation(
            self._get_topic(), callback_func)


class SubModel:
    meta_property = {
        "type": "object",
        "properties": {
            "source": {"type": "string"},
            "submodel_schema": {"type": "object"},
            "submodel_url": {"type": "string"}}
    }

    def __init__(self, asset, submodel_definition):
        # pylint: disable=no-member
        self.parent = asset
        self.communication_client = asset.communication_client
        self.name = sanitize(submodel_definition["name"])

        if asset.access_mode == Mode.OWNER:

            meta_prop = WritableProperty("_meta", self, self.meta_property)

            setattr(self, "_meta", meta_prop)

            if "properties" in submodel_definition:
                self._create_writable_properties(submodel_definition)
            if "operations" in submodel_definition:
                self._create_bindable_operations(submodel_definition)
            if "events" in submodel_definition:
                self._create_triggerable_events(submodel_definition)
        else:

            if "properties" in submodel_definition:
                self._create_readonly_properties(submodel_definition)
            if "operations" in submodel_definition:
                self._create_callable_operations(submodel_definition)
            if "events" in submodel_definition:
                self._create_subscribable_events(submodel_definition)

    def delete(self):
        deletables = [getattr(self, prop) for prop in dir(
            self) if type(getattr(self, prop)) == WritableProperty]
        deletables.extend(getattr(self, prop) for prop in dir(
            self) if type(getattr(self, prop)) == BindableOperation)
        for d in deletables:
            d.delete()

    # noinspection PyProtectedMember
    def _get_topic(self):
        return self.parent._get_topic() + f"/{self.name}"

    def _create_callable_operations(self, submodel_definition):
        for name, schema in submodel_definition["operations"].items():
            new_op = CallableOperation(name, self, schema)
            setattr(self, sanitize(name), new_op.invoke)

    def _create_readonly_properties(self, submodel_definition):
        for name, schema in submodel_definition["properties"].items():
            new_prop = ReadOnlyProperty(name, self, schema)
            setattr(self, sanitize(name), new_prop)

    def _create_writable_properties(self, submodel_definition):
        for name, schema in submodel_definition["properties"].items():
            new_prop = WritableProperty(name, self, schema)
            setattr(self, sanitize(name), new_prop)

    def _create_bindable_operations(self, submodel_definition):
        for name, schema in submodel_definition["operations"].items():
            new_op = BindableOperation(name, self, schema)
            setattr(self, "bind_" + sanitize(name), new_op.bind)

    def _create_subscribable_events(self, submodel_definition):
        for name, schema in submodel_definition["events"].items():
            new_event = SubscribableEvent(name, self, schema)
            setattr(self, sanitize(name), new_event)

    def _create_triggerable_events(self, submodel_definition):
        for name, schema in submodel_definition["events"].items():
            new_event = TriggerableEvent(name, self, schema)
            setattr(self, sanitize(name), new_event.trigger)


class Asset:
    """Core element of assets2036py. Assets are automatically generated during runtime from given submodel descriptions.

    For each submodel description given an asset receives a attribute of the same name.
    Each submodel then implements the operations, properties and events specified by the submodel.

    You shouldn't instanciate assets by yourself, make use of :class: `assets2036py.assetmanager`

    """

    def __init__(self, name: str, namespace: str, *sub_models, mode=Mode.CONSUMER, communication_client, endpoint_name):
        self.name = name
        self.endpoint_name = endpoint_name
        self.namespace = namespace
        self.access_mode = mode
        self.communication_client = communication_client
        self.sub_models = sub_models
        self.sub_model_names = []
        for sm in sub_models:
            self.implement_sub_model(sm)

    def disconnect(self):
        logger.debug("%s disconnects", self.name)
        self.communication_client.disconnect()

    def delete(self):
        for submodel in self.sub_model_names:
            getattr(self, submodel).delete()

    def implement_sub_model(self, submodel):
        if isinstance(submodel, str):
            try:
                # try parsing as json string
                submodel_def = json.loads(submodel)
                submodel_url = "file://localhost"
            except JSONDecodeError:
                # try to load as a web resource
                try:
                    with urlopen(submodel, context=ssl._create_unverified_context()) as response:
                        submodel_def = json.load(response)
                    submodel_url = submodel
                except ValueError as valexc:
                    logger.error(
                        "Could not parse submodel definition of %s:\n%s", submodel, valexc)
                    return
        else:
            submodel_def = submodel
            submodel_url = "file://localhost"
            try:
                jsonschema.validate(submodel_def, submodel_schema)
            except jsonschema.exceptions.ValidationError as valexc:
                logger.error("%s is malformed:\n%s", submodel_def, valexc)

        new_sm = SubModel(self, submodel_def)
        if self.access_mode == Mode.OWNER:
            new_sm._meta.value = {"source": f"{self.namespace}/{self.endpoint_name}",
                                  "submodel_definition": submodel_def,
                                  "submodel_url": submodel_url if submodel_url else "file://localhost"}
        submodel_name = submodel_def['name'].replace("-", "_").replace(" ", "")
        self.sub_model_names.append(submodel_name)
        setattr(self, submodel_name, new_sm)

    def _get_topic(self):
        return f"{self.namespace}/{self.name}"



# Copyright (c) 2016 - for information on the respective copyright owner
# see the NOTICE file and/or the repository https://github.com/boschresearch/assets2036py.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import inspect
import os
import re


def sanitize(s):
    # keep minus sings as underscores
    s = re.sub("-", "_", s)
    # Remove invalid characters
    s = re.sub('[^0-9a-zA-Z_]', '', s)
    # Remove leading characters until we find a letter or underscore
    s = re.sub('^[^a-zA-Z_]+', '', s)
    return s


res_path = os.path.join(os.path.dirname(
    os.path.abspath(__file__)), "resources")


def get_resource_path(filename):
    return os.path.join(res_path, filename)


class Context:
    """ Provide arbitrary data to all called functions
    Usage:
        Set arbitrary property:

        context.set("name","monty")

        All subsequently called functions (in the same stack)
        can access the data directly:

        print(context.name) # outputs "monty"

    """
    class __Context:
        """
        Inner instance class
        """

        def __init__(self):
            self.data = {}

        def __str__(self):
            return repr(self) + self.val

        def __getattr__(self, name):
            frame = inspect.currentframe()
            if name not in self.data:
                return None
            for f in inspect.getouterframes(frame):
                if f.frame in self.data[name]:
                    return self.data[name][f.frame]
            return None

        def set(self, name, val):
            """set or update attribute
            Args:
                name (str): name of the attribute
                val: value of the attribute 
            """
            frame = inspect.getouterframes(inspect.currentframe())[1].frame
            if name not in self.data:
                self.data[name] = {}
            self.data[name][frame] = val

        def free(self):
            frame = inspect.getouterframes(inspect.currentframe())[1].frame
            for var in self.data.copy():
                if frame in self.data[var]:
                    del self.data[var][frame]
                    if not self.data[var]:
                        del self.data[var]

    instance = None

    def __init__(self):
        if not Context.instance:
            Context.instance = Context.__Context()

    def __getattr__(self, name):
        return getattr(self.instance, name)


context = Context()



# Copyright (c) 2016 - for information on the respective copyright owner
# see the NOTICE file and/or the repository https://github.com/boschresearch/assets2036py.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import sys
import uuid
import time
import re
import traceback
from abc import ABC, abstractmethod
import logging
from collections import defaultdict
from queue import Queue, Empty
from concurrent.futures import ThreadPoolExecutor
from datetime import datetime
from dateutil import tz
import paho.mqtt.client as mqtt
from assets2036py.exceptions import OperationTimeoutException, InvalidParameterException
from assets2036py import context


logger = logging.getLogger(__name__)


def create_name_regexp(namespace, assetnname):
    namespace_sanitized = namespace.replace("+", "\w+?")
    assetname_sanitized = assetnname.replace("+", "\w+?")
    return f"{namespace_sanitized}/{assetname_sanitized}/([A-Za-z0-9._-]*)/_meta"


def create_submodel_regexp(namespace, submodelname):
    namespace_sanitized = namespace.replace("+", "\w+?")
    submodelname_sanitized = submodelname.replace("+", "\w+?")
    return f"{namespace_sanitized}/([A-Za-z0-9._-]*)/{submodelname_sanitized}/_meta"


class CommunicationClient(ABC):

    @abstractmethod
    def connect(self, **config):
        pass

    @abstractmethod
    def join(self, timeout=None):
        pass

    @abstractmethod
    def disconnect(self):
        pass

    @abstractmethod
    def publish(self, topic, payload, **config):
        pass

    @abstractmethod
    def subscribe(self, topic, callback):
        pass

    @abstractmethod
    def on_connect(self, cb):
        pass

    @abstractmethod
    def on_disconnect(self, cb):
        pass

    @abstractmethod
    def on_message(self, cb):
        pass

    @abstractmethod
    def run(self):
        pass

    @abstractmethod
    def invoke_operation(self, operation, parameter, timeout):
        pass

    @abstractmethod
    def bind_operation(self, operation, cb):
        pass

    @abstractmethod
    def trigger_event(self, event, parameter):
        pass

    @abstractmethod
    def subscribe_event(self, event, callback):
        pass


class MockClient(CommunicationClient):

    def subscribe_event(self, event, callback):
        pass

    def trigger_event(self, event, parameter):
        pass

    def bind_operation(self, operation, cb):
        pass

    def invoke_operation(self, operation, parameter, timeout):
        pass

    def join(self, timeout=None):
        pass

    def connect(self, **config):
        logger.debug("Connecting with config %s", config)

    def disconnect(self):
        logger.debug("Disconnecting!")

    def publish(self, topic, payload, **config):
        logger.debug("Sending %s to %s with %s",
                     payload, topic, config)

    def subscribe(self, topic, callback):
        logger.debug("Subscribed to %s", topic)

    def on_connect(self, cb):
        pass

    def on_disconnect(self, cb):
        pass

    def on_message(self, cb):
        pass

    def run(self):
        pass


class MQTTClient(CommunicationClient):

    def __init__(self, client_id):
        self._queues = defaultdict(Queue)
        self.client = mqtt.Client(
            f"assets2036py_{client_id}_{uuid.uuid4().hex}", clean_session=False)
        self._executor = ThreadPoolExecutor(max_workers=10)

    def join(self, timeout=None):
        #pylint: disable=protected-access
        self.client._thread.join(timeout)

    def _get_msgs_for_n_secs(self, topic, seconds):
        msgs = defaultdict(list)

        def _message_callback(_client, _userdata, message):
            logger.debug(
                "Callback received %s from %s", message.payload, message.topic)
            msgs[message.topic].append(message.payload)

        self.subscribe(topic, _message_callback)
        time.sleep(seconds)
        self.client.unsubscribe(topic)
        return msgs

    def unsubscribe(self, topic):
        self.client.unsubscribe(topic)

    def trigger_event(self, event, parameter):
        payload = {
            "timestamp": datetime.now(tz.tzlocal()).isoformat(),
            "params": parameter
        }
        self.publish(event, json.dumps(payload), retain=False)

    def query_submodels_for_asset(self, namespace, name, seconds=1):
        msgs = self._get_msgs_for_n_secs(
            f"{namespace}/{name}/+/_meta", seconds)
        submodels = {}
        for topic, meta in msgs.items():
            regexp = create_name_regexp(namespace, name)
            match = re.search(
                regexp, topic)
            if not match:
                continue
            submodel_name = match[1]
            schema = json.loads(meta[0])
            submodels[submodel_name] = schema
        return submodels

    def query_asset_names(self, namespace, *submodel_names, seconds=3):

        asset_names = []
        for sm in submodel_names:

            regexp = create_submodel_regexp(namespace, sm)
            msgs = self._get_msgs_for_n_secs(
                f"{namespace}/+/{sm}/_meta", seconds)
            asset_names_for_sm = set()
            for topic in msgs:
                match = re.search(regexp, topic)
                if not match:
                    continue
                asset_name = match[1]
                asset_names_for_sm.add(
                    asset_name)
            asset_names.append(asset_names_for_sm)
        if not asset_names:
            return set()
        if len(asset_names) == 1:
            return asset_names[0]
        return asset_names[0].intersection(*asset_names[1:])

    def invoke_operation(self, operation, parameter, timeout):
        topic = operation + "/RESP"
        self.subscribe(topic, self._on_response)
        req_id = uuid.uuid4().hex
        payload = {
            "req_id": req_id,
            "params": parameter
        }

        self.publish(operation + "/REQ", json.dumps(payload),
                     retain=False)  # Don't resend method invocations!
        logger.debug("%s invoked with payload %s", operation, payload)
        try:
            start = time.perf_counter()
            response = self._queues[req_id].get(timeout=timeout)
            end = time.perf_counter()
            logger.debug(
                "got response \"%s\" after %s seconds", response, end-start)
            return response
        except Empty:
            # pylint: disable=raise-missing-from
            raise OperationTimeoutException(
                f"{operation} timed out")

    def _on_response(self, payload):
        try:
            payload_obj = json.loads(payload)
            req_id = payload_obj["req_id"]
            self._queues[req_id].put(payload_obj["resp"])
        except KeyError as e:
            logger.error("Received invalid payload: %s", e)
        except json.JSONDecodeError as e:
            logger.error("Received invalid json: %s", e)

    def bind_operation(self, operation, cb):
        def callback_func(payload):
            # pylint: disable=bare-except
            try:
                payload_obj = json.loads(payload)
                req_id = payload_obj["req_id"]
                context.set("req_id", req_id)
                res = cb(payload_obj["params"])
                context.free()
                response = {
                    "req_id": req_id,
                    "resp": res
                }
                self.publish(operation + "/RESP", json.dumps(response),
                             retain=False)  # Don't resend results
            except InvalidParameterException as e:
                logger.error("Callback got invalid parameters: %s", e)
            except KeyError as e:
                logger.error("Received invalid payload: %s", e)
            except json.JSONDecodeError as e:
                logger.error("Received invalid json: %s", e)
            except TypeError as e:
                logger.error(
                    "Callback got wrong number of parameters: %s", e)
            except:
                logger.error(
                    " Exception occured during callback execution:\n %s", traceback.format_exc())

        self.subscribe(
            operation + "/REQ", lambda payload: self._executor.submit(callback_func, payload))

    def subscribe_event(self, event, callback):
        def callback_func(payload):
            # pylint: disable=bare-except
            try:
                payload_obj = json.loads(payload)
                timestamp = payload_obj["timestamp"]
                params = payload_obj["params"]
                callback(params, timestamp)
            except InvalidParameterException as e:
                logger.error("Callback got invalid parameters: %s", e)
            except KeyError as e:
                logger.error("Received invalid payload: %s", e)
            except json.JSONDecodeError as e:
                logger.error("Received invalid json: %s", e)
            except TypeError as e:
                logger.error(
                    "Callback got wrong number of parameters: %s", e)
            except:
                logger.error(
                    " Exception occured during callback execution:\n%s", traceback.format_exc())

        self.subscribe(event, lambda payload: self._executor.submit(
            callback_func, payload))

    def disconnect(self):
        # pylint: disable=protected-access
        if self.client._will_topic:
            # noinspection PyProtectedMember
            self.client.publish(self.client._will_topic.decode(
            ), self.client._will_payload.decode(), retain=True)
        self.client.disconnect()
        self.client.loop_stop()

    def on_connect(self, cb):
        self.client.on_connect = cb

    def on_disconnect(self, cb):
        self.client.on_disconnect = cb

    def on_message(self, cb):
        self.client.on_message = cb

    def connect(self, **config):
        self.client.will_set(
            f"{config['namespace']}/{config['endpoint_name']}/_endpoint/online", "false", retain=True)
        self.client.connect(config["host"], config["port"], keepalive=10)
        self.client.loop_start()

    def publish(self, topic, payload, **config):
        logger.debug("Sending %s to %s", payload, topic)
        self.client.publish(topic, payload, config.get(
            "qos", 2), config.get("retain", True))

    def subscribe(self, topic, callback):

        def callback_func(client, userdata, message):
            logger.debug(
                "Received %s from %s", message.payload, message.topic)
            if message.payload:

                # support callbacks for full info as well as payload only
                if callback.__code__.co_argcount < 3:
                    callback(message.payload)
                else:
                    callback(client, userdata, message)
            else:
                logger.debug("Ignored empty message on %s", message.topic)

        self.client.message_callback_add(
            topic, lambda c, u, m: self._executor.submit(
                callback_func, c, u, m)
        )
        self.client.subscribe(topic, 2)
        logger.debug("Subscribed to %s", topic)

    def run(self):
        pass
        # self.client.loop_start()



# Copyright (c) 2016 - for information on the respective copyright owner
# see the NOTICE file and/or the repository https://github.com/boschresearch/assets2036py.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from os import path
from setuptools import setup, find_packages

here = path.abspath(path.dirname(__file__))


with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='assets2036py',
    version='0.0.14',
    url='https://github.com/boschresearch/assets2036-submodels',
    license='BIOS',
    author='Daniel Ewert (CR/APA3 G6/BD-BBI)',
    author_email='Daniel.Ewert@de.bosch.com',
    description='helper library to easily implement assets2036',
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=find_packages(exclude=['contrib', 'docs', 'tests', 'examples']),
    install_requires=['jsonschema', 'paho-mqtt', "python-dateutil"],
    package_data={"assets2036py": ["resources/*.json"]}
)
